package com.supermap.samplecode.data;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;


import com.supermap.data.CursorType;
import com.supermap.data.DatasetType;
import com.supermap.data.DatasetVector;
import com.supermap.data.DatasetVectorInfo;
import com.supermap.data.DatasourceConnectionInfo;
import com.supermap.data.EngineType;
import com.supermap.data.GeoRegion;
import com.supermap.data.Point2D;
import com.supermap.data.Point2Ds;
import com.supermap.data.Recordset;
import com.supermap.data.Workspace;

/**
 * <p>
 * Title:批量更新
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
 * ------------------------------版权声明----------------------------
 * 此文件为SuperMap Objects Java 的示范代码 
 * 版权所有：北京超图软件股份有限公司
 * ----------------------------------------------------------------
 * ---------------------SuperMap iObjects Java 示范程序说明------------------------
 * 
 * 1、范例简介：批量添加记录
 * 2、示例数据：无； 
 * 3、关键类型/成员: 
 *      Dataset.getRecordset 方法
 *      Recordset.addNew 方法
 *      Recordset.getBatch().setMaxRecordCount 方法
 *      Recordset.getBatch().begin 方法
 *      Recordset.getBatch().update 方法
 *      Recordset.dispose 方法  
 * 4、使用步骤：
 *  (1)阅读说明后，按照菜单提示选择添加记录的模式或退出
 *  (2)示例将按照选择的模式批量添加200个面记录到数据集
 *  (3)程序显示添加所用的时间，并返回到第(1)步
 * ------------------------------------------------------------------------------
 * ============================================================================>
 * </p> 
 * 
 * <p>
 * Company: 北京超图软件股份有限公司
 * </p>
 * 
 */
 
 /**
 * <p>
 * Title:Batch Update
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
 * ------------------------------CopyRight----------------------------
 *This file contants the sample code of SuperMap Objects Java.
 * Copyright @2005-2012 SuperMap Software Co., Ltd. All Rights Reserved. 
 * ----------------------------------------------------------------
 * ---------------------SuperMap Objects Java Sample Code Description------------------------
 * 
 * 1. Sample Code Description: This sample demonstrates the BatchEditor class which provides the batch edit of adding the records 
 * 2. Sample Data: The sample code create the datasource of UDB 
 * 3. Key Classes/Methods:
 *      Dataset.getRecordset 
 *      Recordset.addNew 
 *      Recordset.getBatch().setMaxRecordCount
 *      Recordset.getBatch().begin 
 *      Recordset.getBatch().update 
 *      Recordset.dispose 
 * 4. Procedures
 *  (1)After read the introduction, select the add mode under the introduction of quit the program. 
 *  (2)The program will add 200 region record to the dataset in batch when the add mode is selected add in batch. 
 *  (3)The program will display the time of adding waste and return to the first step. 
 * ------------------------------------------------------------------------------
 * ============================================================================>
 * </p> 
 * 
 * <p>
 * Company: SuperMap Software Co., Ltd
 * </p>
 * 
 */

public class SampleRun {

	private Workspace m_workspace;

	private DatasetVector m_dataset;
	// 要创建的数据库文件名
	// The name of the datasource to be created.
	private static String UDBPATH = "../../SampleData/BatchAdd.udb";

	private static String UDDPATH = "../../SampleData/BatchAdd.udd";
	
	// 查询用到的数据
	// Data fir querying.
	private String m_choice = "";
	private String m_quit = "";
	private String m_commanderror = "";
	private String m_ms = "";
	private String m_print1 = "";
	private String m_print2 = "";
	private String m_print3 = "";
	
	//菜单
	// The menu.

	private static String m_menustring = "";
	
	// 要添加的记录条数
	// The count of the recordset to be added.
	private static int RECORDSETCOUNT = 200;
	
	// 添加状态相关
	// The percent of the adding process.
	private int nowPercent;

	// 开始时间
	// The begin time.
	private long beginTime;
	
	// 结束时间
	// The end time.
	private long endTime;


	/**
	 * 程序入口
	 * The entrance of the programme.
	 */
	public static void main(String[] args) {
		SampleRun sampleRun = new SampleRun();
		sampleRun.run();
		sampleRun.dispose();
	}

	/**
	 * SampleRun 构造函数，创建UDB数据源
	 * The constructor of the SampleRun. Create the UDB datasurce.
	 */
	public SampleRun() {
		try {
			m_workspace = new Workspace();
			
			InitializeCultureResources();
			
			deleteUDBFile();

			DatasourceConnectionInfo connectionInfo = new DatasourceConnectionInfo();
			connectionInfo.setServer(UDBPATH);
			connectionInfo.setEngineType(EngineType.UDB);

			m_workspace.getDatasources().create(connectionInfo);
		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}
	}
	
	/**
	* 初始化多语言资源
	* Initialize CultureResources
	*/
	private void InitializeCultureResources()
	{
		if(com.supermap.data.Environment.getCurrentCulture().contentEquals("zh-CN"))
		{
			m_menustring = "\n请在以下菜单，选择将使用的批量添加的模式：\n1.普通方式添加\n2.批量添加方式添加\nq.退出\n";
			m_choice = "选择：";
			m_quit = "退出...";
			m_commanderror = ":命令错误";
			m_print1 = "添加全部记录成功，所用时间：";
			m_ms = " 毫秒";
			m_print2 = "普通方式记录添加中... ";
			m_print3 = "批量更新记录添加中... ";
			
		}
		else
		{
			m_menustring = "\nPlease choose the batch mode in the menu:\n1.Normal\n2.Batch\nq.Quit\n";
			m_choice = "choose:";
			m_quit = "Quit...";
			m_commanderror = ":Error Command";
			m_print1 = "Add Recordset Successful, Time Cost: ";
			m_ms = " ms";
			m_print2 = "Adding Recordset with Normal Mode ...";
			m_print3 = "Adding Recordset with Batch Mode ...";
			
		}
	}

	/**
	 * SampleRun 运行的主方法
	 * The main method ofSampleRun.
	 */
	public void run() {
		try {
			m_workspace.getDatasources().get(0).getDatasets().delete(
					"TempPoint");
			DatasetVectorInfo datasetInfo = new DatasetVectorInfo();
			datasetInfo.setName("TempPoint");
			datasetInfo.setType(DatasetType.REGION);

			m_dataset = m_workspace.getDatasources().get(0).getDatasets()
					.create(datasetInfo);

			chooseMenuItem();
		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}
	}

	/**
	 * 选择菜单项
	 * Select the menu item.
	 */
	private void chooseMenuItem() {
		System.out.println(m_menustring);
		System.out.print(m_choice);
		switchInputKey();
	}

	/**
	 * 根据输入，做相应的操作
	 * Do the operation according to the input.
	 */
	private void switchInputKey() {
		nowPercent = 0;
		BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
		String input = "";
		try {
			input = in.readLine();
		} catch (IOException e) {
			System.out.println(e.getMessage());
		}

		if (input.equals("1")) {
			printTimeSpan(addPointNormally());
			chooseMenuItem();
		} else if (input.equals("2")) {
			printTimeSpan(addPointInBatch());
			chooseMenuItem();
		} else if (input.equals("q")) {
			System.out.println(m_quit);
			return;
		} else {
			System.out.println(input + m_commanderror);
			chooseMenuItem();
		}

	}

	/**
	 * 打印出时间间隔
	 * Print the time span.
	 */
	private void printTimeSpan(long totalMilliseconds) {
		System.out.print(m_print1);
		System.out.print(totalMilliseconds);
		System.out.print(m_ms);
		System.out.println();
	}

	/**
	 * 普通方式添加记录,返回普通添加需要的时间
	 * Returns the time of the adding process.
	 */
	private long addPointNormally() {
		try {
			System.out.println(m_print2);
			Recordset recordset = m_dataset.getRecordset(false,
					CursorType.DYNAMIC);

			beginTime = System.currentTimeMillis();
			for (int i = 0; i < RECORDSETCOUNT; i++) {
				Point2Ds points = new Point2Ds(new Point2D[] { randomPoint(),
						randomPoint(), randomPoint(), randomPoint() });
				GeoRegion region = new GeoRegion(points);

				recordset.addNew(region);
				recordset.update();
				printWaittingString(i);
			}
			endTime = System.currentTimeMillis();

			// recordset使用完毕后必须释放
			// The recordset must be disposed after using.
			recordset.dispose();
		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}

		return (endTime - beginTime);
	}

	/**
	 * 获得随机点
	 * Gets the random point.
	 */
	private Point2D randomPoint() {
       return new Point2D(Math.random()*100, Math.random()*100);
	}

	/**
	 * 批量更新方式添加记录,返回批量更新需要的时间
	 * Returns the time of the Batch Update process.
	 */
	private long addPointInBatch() {
		try {
			System.out.println(m_print3);
			Recordset recordset = m_dataset.getRecordset(false,
					CursorType.DYNAMIC);

			// 设置批量更新的限度为1000，注意一定要在开始批量更新前设置MaxRecordCount!
			// The maximum record count of Batch Update is 1000. Remember to transfer  setMaxRecordCount before Batch Update.
			recordset.getBatch().setMaxRecordCount(1000);

			// 开始批量更新，当添加到设置的MaxRecordCount的下一条记录时，将会将MaxRecordCount条记录自动提交到数据库中。
			// Begin the Batch Update. When adding the next record of the MaxRecordCount, the MaxRecordCount will be put in to the datssource automatically.
			beginTime = System.currentTimeMillis();
			recordset.getBatch().begin();

			for (int i = 0; i < RECORDSETCOUNT; i++) {
				Point2Ds points = new Point2Ds(new Point2D[] { randomPoint(),
						randomPoint(), randomPoint(), randomPoint() });
				GeoRegion region = new GeoRegion(points);
				recordset.addNew(region);
				printWaittingString(i);
			}

			// 使用批量更新的update，提交没有自动提交的记录
			// Put in the record which is not put in automatically.
			recordset.getBatch().update();

			endTime = System.currentTimeMillis();

			// recordset 使用完毕后必须 释放
			// The recordset ,disposed after using.
			recordset.dispose();
		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}

		return (endTime - beginTime);
	}

	/**
	 * 添加时，添加完成过程中的百分比
	 * Display the percent of the adding process.
	 */
	private void printWaittingString(int index) {
		if ((index + 1) % (RECORDSETCOUNT / 100) == 0) {
			nowPercent++;
			System.out.println(nowPercent + "%");
		}
	}

	/**
	 * 删除创建的UDB与UDD文件，创建之前与程序退出时使用
	 * Delete the created UDB and UDD file. This will be transfered before creating, and when quitting the process.
	 */
	private void deleteUDBFile() {
		try {
			File udbFile = new File(UDBPATH);
			if (udbFile.exists()) {
				udbFile.delete();
			}

			File uddFile = new File(UDDPATH);
			if (uddFile.exists()) {
				uddFile.delete();
			}
		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}
	}

	/**
	 * 释放资源
	 * Dispose the resource.
	 */
	public void dispose() {
		try {
			m_workspace.dispose();
			deleteUDBFile();
		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}
	}
}

